@@ -60,8 +60,6 @@
 
 #define APPROACHING_INFINITY 1000   // Close enough.  This isn't rocket surgery.
 
-extern Task& contextTask;
-
 static const float epsilon = 0.000001;
 #endif
 
@@ -114,6 +112,12 @@ bool Task::operator== (const Task& other)
   return true;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+bool Task::operator!= (const Task& other)
+{
+  return !(*this == other);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Task::Task (const std::string& input)
 {
@@ -140,6 +144,19 @@ Task::Task (const json::object* obj)
   parseJSON (obj);
 }
 
+////////////////////////////////////////////////////////////////////////////////
+Task::Task (tc::Task obj)
+{
+  id               = 0;
+  urgency_value    = 0.0;
+  recalc_urgency   = true;
+  is_blocked       = false;
+  is_blocking      = false;
+  annotation_count = 0;
+
+  parseTC (obj);
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 Task::status Task::textToStatus (const std::string& input)
 {
@@ -182,8 +199,8 @@ const std::string Task::identifier (bool shortened /* = false */) const
 ////////////////////////////////////////////////////////////////////////////////
 void Task::setAsNow (const std::string& att)
 {
-  char now[16];
-  snprintf (now, 16, "%u", (unsigned int) time (nullptr));
+  char now[22];
+  snprintf (now, 22, "%lli", (long long int) time (nullptr));
   set (att, now);
 
   recalc_urgency = true;
@@ -363,6 +380,14 @@ Task::dateState Task::getDateState (const std::string& name) const
   return dateNotDue;
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// An empty task is typically a "dummy", such as in DOM evaluation, which may or
+// may not occur in the context of a task.
+bool Task::is_empty () const
+{
+  return data.size () == 0;
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // Ready means pending, not blocked and either not scheduled or scheduled before
 // now.
@@ -546,9 +571,11 @@ bool Task::is_udaPresent () const
 bool Task::is_orphanPresent () const
 {
   for (auto& att : data)
-    if (att.first.compare (0, 11, "annotation_", 11) != 0)
-      if (Context::getContext ().columns.find (att.first) == Context::getContext ().columns.end ())
-        return true;
+    if (! isAnnotationAttr (att.first) &&
+        ! isTagAttr (att.first) &&
+        ! isDepAttr (att.first) &&
+        Context::getContext ().columns.find (att.first) == Context::getContext ().columns.end ())
+      return true;
 
   return false;
 }
@@ -576,12 +603,14 @@ bool Task::is_overdue () const
 #endif
 
 ////////////////////////////////////////////////////////////////////////////////
+// Task is considered waiting if it's pending and the wait attribute is set as
+// future datetime value.
+// While this is not consistent with other attribute-based virtual tags, such
+// as +BLOCKED, it is more backwards compatible with how +WAITING virtual tag
+// behaved in the past, when waiting had a dedicated status value.
 bool Task::is_waiting () const
 {
-  // note that is_waiting can return true for tasks in an actual status other
-  // than pending; in this case +WAITING will be set but the status will not be
-  // "waiting"
-  if (has ("wait"))
+  if (has ("wait") && get ("status") == "pending")
   {
     Datetime now;
     Datetime wait (get_date ("wait"));
@@ -761,7 +790,25 @@ void Task::parseJSON (const json::object* root_obj)
       else if (i.first == "depends" && i.second->type() == json::j_string)
       {
         auto deps = (json::string*)i.second;
-        auto uuids = split (deps->_data, ',');
+
+        // Fix for issue#2689: taskserver sometimes encodes the depends
+        // property as a string of the format `[\"uuid\",\"uuid\"]`
+        // The string includes the backslash-escaped `"` characters, making
+        // it invalid JSON.  Since we know the characters we're looking for,
+        // we'll just filter out everything else.
+        std::string deps_str = deps->_data;
+        if (deps_str.front () == '[' && deps_str.back () == ']') {
+          std::string filtered;
+		  for (auto &c: deps_str) {
+			if ((c >= '0' && c <= '9') ||
+				(c >= 'a' && c <= 'f') ||
+				c == ',' || c == '-') {
+			  filtered.push_back(c);
+			}
+		  }
+		  deps_str = filtered;
+        }
+        auto uuids = split (deps_str, ',');
 
         for (const auto& uuid : uuids)
           addDependency (uuid);
@@ -861,6 +908,29 @@ void Task::parseJSON (const json::object* root_obj)
   }
 }
 
+////////////////////////////////////////////////////////////////////////////////
+// Note that all fields undergo encode/decode.
+void Task::parseTC (const tc::Task& task)
+{
+  data = task.get_taskmap ();
+
+  // count annotations
+  annotation_count = 0;
+  for (auto i : data)
+  {
+    if (isAnnotationAttr (i.first))
+    {
+      ++annotation_count;
+    }
+  }
+
+  data["uuid"] = task.get_uuid ();
+  id = Context::getContext ().tdb2.id (data["uuid"]);
+
+  is_blocking = task.is_blocking();
+  is_blocked = task.is_blocked();
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // No legacy formats are currently supported as of 2.4.0.
 void Task::parseLegacy (const std::string& line)
@@ -1119,7 +1189,7 @@ void Task::addAnnotation (const std::string& description)
 
   do
   {
-    key = "annotation_" + format ((int) now);
+    key = "annotation_" + format ((long long int) now);
     ++now;
   }
   while (has (key));
@@ -1177,7 +1247,7 @@ void Task::setAnnotations (const std::map <std::string, std::string>& annotation
 void Task::addDependency (int depid)
 {
   // Check that id is resolvable.
-  std::string uuid = Context::getContext ().tdb2.pending.uuid (depid);
+  std::string uuid = Context::getContext ().tdb2.uuid (depid);
   if (uuid == "")
     throw format ("Could not create a dependency on task {1} - not found.", depid);
 
@@ -1225,7 +1295,7 @@ void Task::addDependency (const std::string& uuid)
 ////////////////////////////////////////////////////////////////////////////////
 void Task::removeDependency (int id)
 {
-  std::string uuid = Context::getContext ().tdb2.pending.uuid (id);
+  std::string uuid = Context::getContext ().tdb2.uuid (id);
 
   // The removeDependency(std::string&) method will check this too, but here we
   // can give a more natural error message containing the id provided by the user
@@ -1262,7 +1332,7 @@ std::vector <int> Task::getDependencyIDs () const
     if (!isDepAttr (attr))
       continue;
     auto dep = attr2Dep (attr);
-    ids.push_back (Context::getContext ().tdb2.pending.id (dep));
+    ids.push_back (Context::getContext ().tdb2.id (dep));
   }
 
   return ids;
@@ -1292,7 +1362,7 @@ std::vector <Task> Task::getDependencyTasks () const
   // efficient.
   std::vector <Task> blocking;
   if (uuids.size() > 0)
-    for (auto& it : Context::getContext ().tdb2.pending.get_tasks ())
+    for (auto& it : Context::getContext ().tdb2.pending_tasks ())
       if (it.getStatus () != Task::completed &&
           it.getStatus () != Task::deleted   &&
           std::find (uuids.begin (), uuids.end (), it.get ("uuid")) != uuids.end ())
@@ -1307,7 +1377,7 @@ std::vector <Task> Task::getBlockedTasks () const
   auto uuid = get ("uuid");
 
   std::vector <Task> blocked;
-  for (auto& it : Context::getContext ().tdb2.pending.get_tasks ())
+  for (auto& it : Context::getContext ().tdb2.pending_tasks ())
     if (it.getStatus () != Task::completed &&
         it.getStatus () != Task::deleted   &&
         it.hasDependency (uuid))
@@ -1467,7 +1537,7 @@ void Task::removeTag (const std::string& tag)
 ////////////////////////////////////////////////////////////////////////////////
 void Task::fixTagsAttribute ()
 {
-  // Fix up the old `tags` attribute to match the `tags_..` attributes (or
+  // Fix up the old `tags` attribute to match the `tag_..` attributes (or
   // remove it if there are no tags)
   auto tags = getTags ();
   if (tags.size () > 0) {
@@ -1478,16 +1548,16 @@ void Task::fixTagsAttribute ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Task::isTagAttr(const std::string& attr) const
+bool Task::isTagAttr(const std::string& attr)
 {
-  return attr.compare(0, 5, "tags_") == 0;
+  return attr.compare(0, 4, "tag_") == 0;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
 const std::string Task::tag2Attr (const std::string& tag) const
 {
   std::stringstream tag_attr;
-  tag_attr << "tags_" << tag;
+  tag_attr << "tag_" << tag;
   return tag_attr.str();
 }
 
@@ -1495,7 +1565,7 @@ const std::string Task::tag2Attr (const std::string& tag) const
 const std::string Task::attr2Tag (const std::string& attr) const
 {
   assert (isTagAttr (attr));
-  return attr.substr(5);
+  return attr.substr(4);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -1512,7 +1582,7 @@ void Task::fixDependsAttribute ()
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Task::isDepAttr(const std::string& attr) const
+bool Task::isDepAttr(const std::string& attr)
 {
   return attr.compare(0, 4, "dep_") == 0;
 }
@@ -1533,7 +1603,7 @@ const std::string Task::attr2Dep (const std::string& attr) const
 }
 
 ////////////////////////////////////////////////////////////////////////////////
-bool Task::isAnnotationAttr(const std::string& attr) const
+bool Task::isAnnotationAttr(const std::string& attr)
 {
   return attr.compare(0, 11, "annotation_") == 0;
 }
@@ -1541,7 +1611,7 @@ bool Task::isAnnotationAttr(const std::string& attr) const
 #ifdef PRODUCT_TASKWARRIOR
 ////////////////////////////////////////////////////////////////////////////////
 // A UDA Orphan is an attribute that is not represented in context.columns.
-std::vector <std::string> Task::getUDAOrphanUUIDs () const
+std::vector <std::string> Task::getUDAOrphans () const
 {
   std::vector <std::string> orphans;
   for (auto& it : data)
@@ -1677,6 +1747,9 @@ void Task::substitute (
 //   1) To provide missing attributes where possible
 //   2) To provide suitable warnings about odd states
 //   3) To generate errors when the inconsistencies are not fixable
+//   4) To update status depending on other attributes
+//
+// Critically, note that despite the name this is not a read-only function.
 //
 void Task::validate (bool applyDefault /* = true */)
 {
@@ -1753,7 +1826,7 @@ void Task::validate (bool applyDefault /* = true */)
   if ((status == Task::pending) && (get ("end") != ""))
     remove ("end");
 
-  // Provide an entry date unless user already specified one.
+  // Provide a modified date unless user already specified one.
   if (! has ("modified") || get ("modified") == "")
     setAsNow ("modified");
 
@@ -1904,6 +1977,31 @@ const std::string Task::decode (const std::string& value) const
   return          str_replace (modified, "&close;", "]");
 }
 
+////////////////////////////////////////////////////////////////////////////////
+tc::Status Task::status2tc (const Task::status status)
+{
+  switch (status) {
+    case Task::pending: return tc::Status::Pending;
+    case Task::completed: return tc::Status::Completed;
+    case Task::deleted: return tc::Status::Deleted;
+    case Task::waiting: return tc::Status::Pending; // waiting is no longer a status
+    case Task::recurring: return tc::Status::Recurring;
+    default: return tc::Status::Unknown;
+  }
+}
+
+////////////////////////////////////////////////////////////////////////////////
+Task::status Task::tc2status (const tc::Status status)
+{
+  switch (status) {
+    case tc::Status::Pending: return Task::pending;
+    case tc::Status::Completed: return Task::completed;
+    case tc::Status::Deleted: return Task::deleted;
+    case tc::Status::Recurring: return Task::recurring;
+    default: return Task::pending;
+  }
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 int Task::determineVersion (const std::string& line)
 {
@@ -2107,7 +2205,7 @@ float Task::urgency ()
 ////////////////////////////////////////////////////////////////////////////////
 float Task::urgency_inherit () const
 {
-  float v = FLT_MIN;
+  float v = -FLT_MAX;
 #ifdef PRODUCT_TASKWARRIOR
   // Calling getBlockedTasks is rather expensive.
   // It is called recursively for each dependency in the chain here.
@@ -2253,6 +2351,10 @@ void Task::modify (modType type, bool text_required /* = false */)
 {
   std::string label = "  [1;37;43mMODIFICATION[0m ";
 
+  // while reading the parse tree, consider DOM references in the context of
+  // this task
+  auto currentTask = Context::getContext ().withCurrentTask(this);
+
   // Need this for later comparison.
   auto originalStatus = getStatus ();
 
@@ -2272,6 +2374,19 @@ void Task::modify (modType type, bool text_required /* = false */)
             value == "''"   ||
             value == "\"\"")
         {
+          // Special case: Handle bulk removal of 'tags' and 'depends" virtual
+          // attributes
+          if (name == "depends")
+          {
+            for (auto dep: getDependencyUUIDs ())
+              removeDependency(dep);
+          }
+          else if (name == "tags")
+          {
+            for (auto tag: getTags ())
+              removeTag(tag);
+          }
+
           // ::composeF4 will skip if the value is blank, but the presence of
           // the attribute will prevent ::validate from applying defaults.
           if ((has (name) && get (name) != "") ||
@@ -2400,7 +2515,8 @@ void Task::modify (modType type, bool text_required /* = false */)
 #endif
 
 ////////////////////////////////////////////////////////////////////////////////
-// Compare this task to another and summarize the differences for display
+// Compare this task to another and summarize the differences for display, in
+// the future tense ("Foo will be set to ..").
 std::string Task::diff (const Task& after) const
 {
   // Attributes are all there is, so figure the different attribute names
@@ -2645,3 +2761,206 @@ std::string Task::diffForInfo (
 }
 
 ////////////////////////////////////////////////////////////////////////////////
+// Similar to diff, but formatted as a side-by-side table for an Undo preview
+Table Task::diffForUndoSide (
+  const Task& after) const
+{
+  // Set the colors.
+  Color color_red   (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.before") : "");
+  Color color_green (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.after") : "");
+
+  // Attributes are all there is, so figure the different attribute names
+  // between before and after.
+  Table view;
+  view.width (Context::getContext ().getWidth ());
+  view.intraPadding (2);
+  view.add ("");
+  view.add ("Prior Values");
+  view.add ("Current Values");
+  setHeaderUnderline (view);
+
+  if (!is_empty ())
+  {
+    const Task &before = *this;
+
+    std::vector <std::string> beforeAtts;
+    for (auto& att : before.data)
+      beforeAtts.push_back (att.first);
+
+    std::vector <std::string> afterAtts;
+    for (auto& att : after.data)
+      afterAtts.push_back (att.first);
+
+    std::vector <std::string> beforeOnly;
+    std::vector <std::string> afterOnly;
+    listDiff (beforeAtts, afterAtts, beforeOnly, afterOnly);
+
+    int row;
+    for (auto& name : beforeOnly)
+    {
+      row = view.addRow ();
+      view.set (row, 0, name);
+      view.set (row, 1, renderAttribute (name, before.get (name)), color_red);
+    }
+
+    for (auto& att : before.data)
+    {
+      std::string priorValue   = before.get (att.first);
+      std::string currentValue = after.get  (att.first);
+
+      if (currentValue != "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, att.first);
+        view.set (row, 1, renderAttribute (att.first, priorValue),
+                  (priorValue != currentValue ? color_red : Color ()));
+        view.set (row, 2, renderAttribute (att.first, currentValue),
+                  (priorValue != currentValue ? color_green : Color ()));
+      }
+    }
+
+    for (auto& name : afterOnly)
+    {
+      row = view.addRow ();
+      view.set (row, 0, name);
+      view.set (row, 2, renderAttribute (name, after.get (name)), color_green);
+    }
+  }
+  else
+  {
+    int row;
+    for (auto& att : after.data)
+    {
+      row = view.addRow ();
+      view.set (row, 0, att.first);
+      view.set (row, 2, renderAttribute (att.first, after.get (att.first)), color_green);
+    }
+  }
+
+  return view;
+}
+
+////////////////////////////////////////////////////////////////////////////////
+// Similar to diff, but formatted as a diff for an Undo preview
+Table Task::diffForUndoPatch (
+  const Task& after,
+  const Datetime& lastChange) const
+{
+  // This style looks like this:
+  //  --- before    2009-07-04 00:00:25.000000000 +0200
+  //  +++ after    2009-07-04 00:00:45.000000000 +0200
+  //
+  // - name: old           // att deleted
+  // + name:
+  //
+  // - name: old           // att changed
+  // + name: new
+  //
+  // - name:
+  // + name: new           // att added
+  //
+
+  // Set the colors.
+  Color color_red   (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.before") : "");
+  Color color_green (Context::getContext ().color () ? Context::getContext ().config.get ("color.undo.after") : "");
+
+  const Task &before = *this;
+
+  // Generate table header.
+  Table view;
+  view.width (Context::getContext ().getWidth ());
+  view.intraPadding (2);
+  view.add ("");
+  view.add ("");
+
+  int row = view.addRow ();
+  view.set (row, 0, "--- previous state", color_red);
+  view.set (row, 1, "Undo will restore this state", color_red);
+
+  row = view.addRow ();
+  view.set (row, 0, "+++ current state ", color_green);
+  view.set (row, 1, format ("Change made {1}",
+                            lastChange.toString (Context::getContext ().config.get ("dateformat"))),
+                    color_green);
+
+  view.addRow ();
+
+  // Add rows to table showing diffs.
+  std::vector <std::string> all = Context::getContext ().getColumns ();
+
+  // Now factor in the annotation attributes.
+  for (auto& it : before.data)
+    if (it.first.substr (0, 11) == "annotation_")
+      all.push_back (it.first);
+
+  for (auto& it : after.data)
+    if (it.first.substr (0, 11) == "annotation_")
+      all.push_back (it.first);
+
+  // Now render all the attributes.
+  std::sort (all.begin (), all.end ());
+
+  std::string before_att;
+  std::string after_att;
+  std::string last_att;
+  for (auto& a : all)
+  {
+    if (a != last_att)  // Skip duplicates.
+    {
+      last_att = a;
+
+      before_att = before.get (a);
+      after_att  = after.get (a);
+
+      // Don't report different uuid.
+      // Show nothing if values are the unchanged.
+      if (a == "uuid" ||
+          before_att == after_att)
+      {
+        // Show nothing - no point displaying that which did not change.
+
+        // row = view.addRow ();
+        // view.set (row, 0, *a + ":");
+        // view.set (row, 1, before_att);
+      }
+
+      // Attribute deleted.
+      else if (before_att != "" && after_att == "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+        view.set (row, 1, before_att, color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+      }
+
+      // Attribute added.
+      else if (before_att == "" && after_att != "")
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+        view.set (row, 1, after_att, color_green);
+      }
+
+      // Attribute changed.
+      else
+      {
+        row = view.addRow ();
+        view.set (row, 0, '-' + a + ':', color_red);
+        view.set (row, 1, before_att, color_red);
+
+        row = view.addRow ();
+        view.set (row, 0, '+' + a + ':', color_green);
+        view.set (row, 1, after_att, color_green);
+      }
+    }
+  }
+
+  return view;
+}
+
+////////////////////////////////////////////////////////////////////////////////
