@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -171,7 +171,7 @@ void ColumnDescription::render (
     {
       for (const auto& i : task.getAnnotations ())
       {
-        Datetime dt (strtol (i.first.substr (11).c_str (), nullptr, 10));
+        Datetime dt (strtoll (i.first.substr (11).c_str (), nullptr, 10));
         description += '\n' + std::string (_indent, ' ') + dt.toString (_dateformat) + ' ' + i.second;
       }
     }
@@ -200,7 +200,7 @@ void ColumnDescription::render (
     {
       for (const auto& i : task.getAnnotations ())
       {
-        Datetime dt (strtol (i.first.substr (11).c_str (), nullptr, 10));
+        Datetime dt (strtoll (i.first.substr (11).c_str (), nullptr, 10));
         description += ' ' + dt.toString (_dateformat) + ' ' + i.second;
       }
     }
