@@ -1,7 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
-// taskwarrior - a command line task list manager.
 //
-// Copyright 2006-2013, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -21,43 +20,24 @@
 // OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 // SOFTWARE.
 //
-// http://www.opensource.org/licenses/mit-license.php
+// https://www.opensource.org/licenses/mit-license.php
 //
 ////////////////////////////////////////////////////////////////////////////////
 
-#define L10N                                           // Localization complete.
-
-#include <math.h>
-#include <Context.h>
+#include <cmake.h>
 #include <ColID.h>
-#include <text.h>
-#include <i18n.h>
-
-extern Context context;
+#include <math.h>
+#include <format.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 ColumnID::ColumnID ()
 {
   _name       = "id";
-  _type       = "number";
   _style      = "number";
-  _label      = STRING_COLUMN_LABEL_ID;
+  _label      = "ID";
   _modifiable = false;
-
-  _styles.push_back ("number");
-
-  _examples.push_back ("123");
-}
-
-////////////////////////////////////////////////////////////////////////////////
-ColumnID::~ColumnID ()
-{
-}
-
-////////////////////////////////////////////////////////////////////////////////
-bool ColumnID::validate (std::string& value)
-{
-  return true;
+  _styles     = {"number"};
+  _examples   = {"123"};
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -66,17 +46,14 @@ void ColumnID::measure (Task& task, unsigned int& minimum, unsigned int& maximum
 {
   int length;
 
-       if (task.id < 10)     length = 1;                              // Fast
-  else if (task.id < 100)    length = 2;                              // Fast
-  else if (task.id < 1000)   length = 3;                              // Fast
-  else if (task.id < 10000)  length = 4;                              // Fast
-  else                       length = (int) log10 ((double) task.id); // Slow
+       if (task.id < 10)     length = 1;                                  // Fast
+  else if (task.id < 100)    length = 2;                                  // Fast
+  else if (task.id < 1000)   length = 3;                                  // Fast
+  else if (task.id < 10000)  length = 4;                                  // Fast
+  else if (task.id < 100000) length = 5;                                  // Fast
+  else                       length = 1 + (int) log10 ((double) task.id); // Slow
 
   minimum = maximum = length;
-
-  if (_style != "default" &&
-      _style != "number")
-    throw format (STRING_COLUMN_BAD_FORMAT, _name, _style);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -86,10 +63,11 @@ void ColumnID::render (
   int width,
   Color& color)
 {
+   // Completed and deleted tasks have no ID.
   if (task.id)
-    lines.push_back (color.colorize (rightJustify (task.id, width)));
+    renderInteger (lines, width, color, task.id);
   else
-    lines.push_back (color.colorize (rightJustify ("-", width)));
+    renderStringRight (lines, width, color, "-");
 }
 
 ////////////////////////////////////////////////////////////////////////////////
