@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -107,7 +107,7 @@ int CmdImport::import (const std::string& input)
       if (root->type () == json::j_object)
       {
         // For each object element...
-        json::object* root_obj = (json::object*)root;
+        auto root_obj = (json::object*)root;
         importSingleTask (root_obj);
         ++count;
       }
@@ -116,13 +116,13 @@ int CmdImport::import (const std::string& input)
       //   [ { ... } , { ... } ]
       else if (root->type () == json::j_array)
       {
-        json::array* root_arr = (json::array*)root;
+        auto root_arr = (json::array*)root;
 
         // For each object element...
         for (auto& element : root_arr->_data)
         {
           // For each object element...
-          json::object* root_obj = (json::object*)element;
+          auto root_obj = (json::object*)element;
           importSingleTask (root_obj);
           ++count;
         }
@@ -133,7 +133,7 @@ int CmdImport::import (const std::string& input)
   }
 
   // If an exception is caught, then it is because the free-form JSON
-  // objects/array above failed to parse. This is an indication that the input
+  // objects/array above failed to parse. This may be an indication that the input
   // is an old-style line-by-line set of JSON objects, because both an array of
   // objects, and a single object have failed to parse..
   //
@@ -142,19 +142,30 @@ int CmdImport::import (const std::string& input)
   //   { ... }
   catch (std::string& e)
   {
+    // Make a very cursory check for the old-style format.
+    if (input[0] != '{') {
+      throw e;
+    }
+
+    // Read the entire file, so that errors do not result in a partial import.
+    std::vector<std::unique_ptr<json::object>> objects;
     for (auto& line : split (input, '\n'))
     {
       if (line.length ())
       {
         json::value* root = json::parse (line);
-        if (root)
-        {
-          importSingleTask ((json::object*) root);
-          ++count;
-          delete root;
-        }
+        if (root && root->type () == json::j_object)
+          objects.push_back (std::unique_ptr<json::object> ((json::object *)root));
+        else
+          throw format ("Invalid JSON: {1}", line);
       }
     }
+
+    // Import the tasks.
+    for (auto& root : objects) {
+      importSingleTask (root.get());
+      ++count;
+    }
   }
 
   return count;
@@ -198,7 +209,7 @@ void CmdImport::importSingleTask (json::object* obj)
     if (hasGeneratedEnd)
       task.set ("end", before.get ("end"));
 
-    if (before.data != task.data)
+    if (before != task)
     {
       CmdModify modHelper;
       modHelper.checkConsistency (before, task);
