@@ -1,6 +1,6 @@
 ////////////////////////////////////////////////////////////////////////////////
 //
-// Copyright 2006 - 2020, Paul Beckingham, Federico Hernandez.
+// Copyright 2006 - 2021, Tomas Babej, Paul Beckingham, Federico Hernandez.
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,7 @@
 #include <shared.h>
 #include <format.h>
 #include <util.h>
+#include <Lexer.h>
 
 ////////////////////////////////////////////////////////////////////////////////
 CmdInfo::CmdInfo ()
@@ -76,11 +77,6 @@ int CmdInfo::execute (std::string& output)
     rc = 1;
   }
 
-  // Get the undo data.
-  std::vector <std::string> undo;
-  if (Context::getContext ().config.getBoolean ("journal.info"))
-    undo = Context::getContext ().tdb2.undo.get_lines ();
-
   // Determine the output date format, which uses a hierarchy of definitions.
   //   rc.dateformat.info
   //   rc.dateformat
@@ -147,7 +143,7 @@ int CmdInfo::execute (std::string& output)
 
     // dependencies: blocked
     {
-      auto blocked = dependencyGetBlocking (task);
+      auto blocked = task.getDependencyTasks ();
       if (blocked.size ())
       {
         std::stringstream message;
@@ -162,7 +158,7 @@ int CmdInfo::execute (std::string& output)
 
     // dependencies: blocking
     {
-      auto blocking = dependencyGetBlocked (task);
+      auto blocking = task.getBlockedTasks ();
       if (blocking.size ())
       {
         std::stringstream message;
@@ -184,6 +180,7 @@ int CmdInfo::execute (std::string& output)
     }
 
     // parent
+    // 2017-01-07: Deprecated in 2.6.0
     if (task.has ("parent"))
     {
       row = view.addRow ();
@@ -192,6 +189,7 @@ int CmdInfo::execute (std::string& output)
     }
 
     // mask
+    // 2017-01-07: Deprecated in 2.6.0
     if (task.has ("mask"))
     {
       row = view.addRow ();
@@ -200,6 +198,7 @@ int CmdInfo::execute (std::string& output)
     }
 
     // imask
+    // 2017-01-07: Deprecated in 2.6.0
     if (task.has ("imask"))
     {
       row = view.addRow ();
@@ -207,6 +206,30 @@ int CmdInfo::execute (std::string& output)
       view.set (row, 1, task.get ("imask"));
     }
 
+    // template
+    if (task.has ("template"))
+    {
+      row = view.addRow ();
+      view.set (row, 0, "Template task");
+      view.set (row, 1, task.get ("template"));
+    }
+
+    // last
+    if (task.has ("last"))
+    {
+      row = view.addRow ();
+      view.set (row, 0, "Last instance");
+      view.set (row, 1, task.get ("last"));
+    }
+
+    // rtype
+    if (task.has ("rtype"))
+    {
+      row = view.addRow ();
+      view.set (row, 0, "Recurrence type");
+      view.set (row, 1, task.get ("rtype"));
+    }
+
     // entry
     row = view.addRow ();
     view.set (row, 0, "Entered");
@@ -217,7 +240,7 @@ int CmdInfo::execute (std::string& output)
     auto created = task.get ("entry");
     if (created.length ())
     {
-      Datetime dt (strtol (created.c_str (), nullptr, 10));
+      Datetime dt (strtoll (created.c_str (), nullptr, 10));
       age = Duration (now - dt).formatVague ();
     }
 
@@ -302,16 +325,17 @@ int CmdInfo::execute (std::string& output)
       if (task.hasTag ("ANNOTATED")) virtualTags += "ANNOTATED ";
       if (task.hasTag ("BLOCKED"))   virtualTags += "BLOCKED ";
       if (task.hasTag ("BLOCKING"))  virtualTags += "BLOCKING ";
-      if (task.hasTag ("CHILD"))     virtualTags += "CHILD ";
+      if (task.hasTag ("CHILD"))     virtualTags += "CHILD ";          // 2017-01-07: Deprecated in 2.6.0
       if (task.hasTag ("COMPLETED")) virtualTags += "COMPLETED ";
       if (task.hasTag ("DELETED"))   virtualTags += "DELETED ";
       if (task.hasTag ("DUE"))       virtualTags += "DUE ";
-      if (task.hasTag ("DUETODAY"))  virtualTags += "DUETODAY ";
+      if (task.hasTag ("DUETODAY"))  virtualTags += "DUETODAY ";      // 2016-03-29: Deprecated in 2.6.0
+      if (task.hasTag ("INSTANCE"))  virtualTags += "INSTANCE ";
       if (task.hasTag ("LATEST"))    virtualTags += "LATEST ";
       if (task.hasTag ("MONTH"))     virtualTags += "MONTH ";
       if (task.hasTag ("ORPHAN"))    virtualTags += "ORPHAN ";
       if (task.hasTag ("OVERDUE"))   virtualTags += "OVERDUE ";
-      if (task.hasTag ("PARENT"))    virtualTags += "PARENT ";
+      if (task.hasTag ("PARENT"))    virtualTags += "PARENT ";         // 2017-01-07: Deprecated in 2.6.0
       if (task.hasTag ("PENDING"))   virtualTags += "PENDING ";
       if (task.hasTag ("PRIORITY"))  virtualTags += "PRIORITY ";
       if (task.hasTag ("PROJECT"))   virtualTags += "PROJECT ";
@@ -319,6 +343,7 @@ int CmdInfo::execute (std::string& output)
       if (task.hasTag ("READY"))     virtualTags += "READY ";
       if (task.hasTag ("SCHEDULED")) virtualTags += "SCHEDULED ";
       if (task.hasTag ("TAGGED"))    virtualTags += "TAGGED ";
+      if (task.hasTag ("TEMPLATE"))  virtualTags += "TEMPLATE ";
       if (task.hasTag ("TODAY"))     virtualTags += "TODAY ";
       if (task.hasTag ("TOMORROW"))  virtualTags += "TOMORROW ";
       if (task.hasTag ("UDA"))       virtualTags += "UDA ";
@@ -328,7 +353,7 @@ int CmdInfo::execute (std::string& output)
       if (task.hasTag ("WEEK"))      virtualTags += "WEEK ";
       if (task.hasTag ("YEAR"))      virtualTags += "YEAR ";
       if (task.hasTag ("YESTERDAY")) virtualTags += "YESTERDAY ";
-      // If you update the above list, update src/commands/CmdInfo.cpp and src/commands/CmdTags.cpp as well.
+      // If you update the above list, update src/Task.cpp and src/commands/CmdTags.cpp as well.
 
       row = view.addRow ();
       view.set (row, 0, "Virtual tags");
@@ -344,11 +369,10 @@ int CmdInfo::execute (std::string& output)
     // Task::urgency
     row = view.addRow ();
     view.set (row, 0, "Urgency");
-    view.set (row, 1, format (task.urgency (), 4, 4));
+    view.set (row, 1, Lexer::trimLeft (format (task.urgency (), 4, 4)));
 
     // Show any UDAs
     auto all = task.all ();
-    std::string type;
     for (auto& att: all)
     {
       if (Context::getContext ().columns.find (att) != Context::getContext ().columns.end ())
@@ -362,9 +386,9 @@ int CmdInfo::execute (std::string& output)
             row = view.addRow ();
             view.set (row, 0, col->label ());
 
-            if (type == "date")
+            if (col->type () == "date")
               value = Datetime (value).toString (dateformat);
-            else if (type == "duration")
+            else if (col->type () == "duration")
             {
               Duration iso;
               std::string::size_type cursor = 0;
@@ -385,6 +409,8 @@ int CmdInfo::execute (std::string& output)
     for (auto& att : all)
     {
       if (att.substr (0, 11) != "annotation_" &&
+          att.substr (0, 5) != "tags_" &&
+          att.substr (0, 4) != "dep_" &&
           Context::getContext ().columns.find (att) == Context::getContext ().columns.end ())
       {
          row = view.addRow ();
@@ -440,8 +466,12 @@ int CmdInfo::execute (std::string& output)
               (end = var.first.find (".coefficient")) != std::string::npos)
           {
             auto project = var.first.substr (21, end - 21);
-            if (task.get ("project").find (project) == 0)
+            const std::string taskProjectName = task.get("project");
+            if (taskProjectName == project ||
+                taskProjectName.find(project + '.') == 0)
+            {
               urgencyTerm (urgencyDetails, "PROJECT " + project, 1.0, var.second);
+            }
           }
 
           // urgency.user.tag.<tag>.coefficient
@@ -472,7 +502,7 @@ int CmdInfo::execute (std::string& output)
           if (end != std::string::npos)
           {
             auto uda = var.first.substr (12, end - 12);
-            auto dot = uda.find (".");
+            auto dot = uda.find ('.');
             if (dot == std::string::npos)
             {
               // urgency.uda.<name>.coefficient
@@ -495,54 +525,6 @@ int CmdInfo::execute (std::string& output)
       urgencyDetails.set (row, 5, rightJustify (format (task.urgency (), 4, 4), 6));
     }
 
-    // Create a third table, containing undo log change details.
-    Table journal;
-    setHeaderUnderline (journal);
-
-    if (Context::getContext ().config.getBoolean ("obfuscate"))
-      journal.obfuscate ();
-    if (Context::getContext ().config.getBoolean ("color"))
-      journal.forceColor ();
-
-    journal.width (Context::getContext ().getWidth ());
-    journal.add ("Date");
-    journal.add ("Modification");
-
-    if (Context::getContext ().config.getBoolean ("journal.info") &&
-        undo.size () > 3)
-    {
-      // Scan the undo data for entries matching this task, without making
-      // copies.
-      unsigned int i = 0;
-      long last_timestamp = 0;
-      while (i < undo.size ())
-      {
-        int when = i++;
-        int previous = -1;
-
-        if (! undo[i].compare (0, 3, "old", 3))
-          previous = i++;
-
-        int current = i++;
-        i++; // Separator
-
-        if (undo[current].find ("uuid:\"" + uuid) != std::string::npos)
-        {
-          if (previous != -1)
-          {
-            int row = journal.addRow ();
-
-            Datetime timestamp (strtol (undo[when].substr (5).c_str (), nullptr, 10));
-            journal.set (row, 0, timestamp.toString (dateformat));
-
-            Task before (undo[previous].substr (4));
-            Task after (undo[current].substr (4));
-            journal.set (row, 1, taskInfoDifferences (before, after, dateformat, last_timestamp, Datetime(after.get("modified")).toEpoch()));
-          }
-        }
-      }
-    }
-
     out << optionalBlankLine ()
         << view.render ()
         << '\n';
@@ -550,10 +532,6 @@ int CmdInfo::execute (std::string& output)
     if (urgencyDetails.rows () > 0)
       out << urgencyDetails.render ()
           << '\n';
-
-    if (journal.rows () > 0)
-      out << journal.render ()
-          << '\n';
   }
 
   output = out.str ();
